/*************************************************************************
 *
 *   Program:  NRF24L01_Processor 				Version: 1.00
 *
 *   Author:  SS								Date:  Dec 19, 2014
 *
 *   This program implements functions to control a NRF24L01 wireless peripheral attached to the Nano board.
 *   The NRF24L01 is a high speed wireless module running at 2.4G with 127 channels, 5 byte address setting
 *   per channel and 32 byte packet payload size.  Functions are provided to allow the NRF24L01 to operate 
 *   as a sender, receiver or transceiver.  An enumeration functions checks to see how many devices are in 
 *   as specific channel setting.  The idea is to provide all the necessary low level calls to create a multi-
 *   node wireless network (although no Mesh network is provided).
 *
 *   This project is licensed under the MIT license.  You can modify, enhance, etc anyway you like.
 *
 *   SOC Robotics, Inc is not liable in anyway for the use of this software and assumes no responsibility 
 *   for it's use.  No warranties of any kind are provided.
 *
 *   Copyright 2014, SOC Robotics, Inc.  
 *   There maybe code from other parties in this application  If we don't mention them by name thank you
 *   and if we should please let us know and we will include your name and attribution.
 *
 *************************************************************************/
//#include "hw_config.h"

#include "stdint.h"
#include "stdbool.h"
#include "stdio.h"
#include "math.h"
#include "string.h"
#include "stddef.h"
#include "stdlib.h"
#include "ctype.h"

#include "NRF24L01.h"
#include "FXOS8700.h"
#include "NanoMP_Defines.h"
#include "NanoMP_Externals.h"
#include "colors.h"


extern volatile struct sid {
	unsigned int rawtemperature,rawhumidity;
	float temperature,humidity;
} si7013;

extern FXOS_F dip[];
extern FXOS_F pip[];
extern FXOS_F mcp[];

/***************************************************
  RF24L01 Functions
***************************************************/

volatile uint8_t TADDR[5] = {'S', 'O', 'C', DEVICE_CLASS, DEVICE_ID}; // device address
volatile unsigned char payload[32],rf_state,rf_mode,device_id,rf_payload_size,channel_no,rf_device_class;
volatile char rfbuf[32];
volatile bool rf_inmessage;

/*!
 * \brief Initializes the transceiver.
 */
void RF24_Init(void) 
{
	RF_CE_LOW();   // CE low: do not send or receive data
	RF_CSN_HIGH(); // CSN high: not sending commands to the device
}

/*!
* \brief Writes a byte and reads the value
* \param val Value to write. This value will be shifted out
* \return The value shifted in
*/
volatile static uint8_t SPI_WriteRead(volatile uint8_t val) 
{
	return SPI2_transfer_byte(val);
}

/*!
 * \brief Write a register value to the transceiver
 * \param reg Register to write
 * \param val Value of the register to write
 */
void RF_WriteRegister(volatile uint8_t reg, volatile uint8_t val) 
{	
	//printf("writeregister:%x %x\n\r",reg,val);
	RF_CSN_LOW(); // initiate command sequence

	SPI2_transfer_byte(RF24_W_REGISTER|reg);
	SPI2_transfer_byte(val);

	RF_CSN_HIGH(); // end command sequence
	delay_us(10); // insert a delay until next command
}

unsigned char RF_ReadRegister(volatile uint8_t reg)
{
	volatile unsigned char inchar;
	
	RF_CSN_LOW(); 
	SPI2_transfer_byte(RF24_R_REGISTER|reg);
	inchar = SPI2_transfer_byte(0xFF);
	RF_CSN_HIGH();
	delay_us(10);
	return inchar;
}

// Assumes CSN pulled low
void RF_Write(volatile unsigned char data)
{
	SPI2_transfer_byte(data);
}

/*!
* \brief Read and return the STATUS
* \return Status
*/
volatile uint8_t RF_GetStatus(void) 
{
	volatile unsigned char status;
	
	RF_CSN_LOW();
	status = SPI_WriteRead(RF24_R_REGISTER|RF24_STATUS);
	RF_CSN_HIGH();
	
	return status;
}

/*!
* \brief Read multiple bytes from the bus.
* \param reg Register address
* \param buf Buffer where to write the data
* \param bufSize Buffer size in bytes
*/
void RF_ReadRegisterData(volatile uint8_t reg, uint8_t *buf, volatile uint8_t bufSize)
{
	volatile int i;
	
	RF_CSN_LOW();
	SPI_WriteRead(RF24_R_REGISTER|reg);
	
	for(i=0; i<bufSize; i++) buf[i] = SPI_WriteRead(0xff) ;
	
	RF_CSN_HIGH();
	delay_us(10);
}

/*!
* \brief Write multiple bytes from the bus.
* \param reg Register address
* \param buf Buffer where to write the data
* \param bufSize Buffer size in bytes
*/
void RF_WriteRegisterData(volatile uint8_t reg, uint8_t *buf, volatile uint8_t bufSize)
{
	volatile int i;
	
	RF_CSN_LOW();
	SPI_WriteRead(RF24_W_REGISTER|reg);
	
	for(i=0; i<bufSize; i++) {
		SPI_WriteRead(buf[i]);
		//if(buf[i]!=0) printf("%c",buf[i]);
		}
	
	RF_CSN_HIGH();
	delay_us(10);
}

/*
** ===================================================================
**     Event       :  IRQ_OnInterrupt (module Events)
**
**     Component   :  IRQ [ExtInt]
**     Description :
**         This event is called when an active signal edge/level has
**         occurred.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
static volatile bool isrFlag; // flag set by ISR

void APP_OnInterrupt(void)
{
	//CE_ClrVal(); // stop sending/listening
	isrFlag = true;
}

void RF24_Payload_Channel(volatile unsigned char payloadsize,volatile unsigned char channel)
{
	rf_payload_size = payloadsize;
	RF_WriteRegister(RF24_RX_PW_P0, payloadsize); // number of payload bytes we want to send and receive
	channel_no = channel;
	RF_WriteRegister(RF24_RF_CH, channel); // set channel
}

void RF24L01_Processor_Init(volatile char rftype,volatile uint8_t payloadsize,volatile uint8_t channel)
{	
	RF24_Init(); // set CE and CSN to initialization value
	//RF_WriteRegister(RF24_RF_SETUP, RF24_RF_SETUP_RF_PWR_0|RF24_RF_SETUP_RF_DR_250);
	//RF_WriteRegister(RF24_RF_SETUP, RF24_RF_SETUP_RF_PWR_0|RF24_RF_SETUP_LNA_HCURR|RF24_RF_SETUP_RF_DR_1000);
	
	RF24_Payload_Channel(payloadsize,channel);

	// Set RADDR and TADDR as the transmit address since we also enable auto acknowledgment
	TADDR[4] = device_id;
	RF_WriteRegisterData(RF24_RX_ADDR_P0, (uint8_t*)TADDR, 5);
	RF_WriteRegisterData(RF24_TX_ADDR, (uint8_t*)TADDR, 5);

	// Enable RX_ADDR_P0 address matching
	//RF_WriteRegister(RF24_EN_RXADDR, RF24_EN_RXADDR_ERX_P0); // enable data pipe 0
	
	RF_WriteRegister(RF24_SETUP_RETR, RF24_SETUP_RETR_ARD_750|RF24_SETUP_RETR_ARC_6); // Important: need 750 us delay between every retry

	if(rftype==RFSENDER) {
		//printf("RF24L01 sender enabled\n\r");
		//RF_WriteRegister(RF24_EN_AA, RF24_EN_AA_ENAA_P0); // enable auto acknowledge. RX_ADDR_P0 needs to be equal to TX_ADDR!
		//RF_WriteRegister(RF24_SETUP_RETR, RF24_SETUP_RETR_ARD_750|RF24_SETUP_RETR_ARC_0); // Important: need 750 us delay - no retry
		
		TX_POWERUP();  // Power up in transmitting mode
		delay_us(150);
		//CE_ClrVal();   // Will pulse this later to send data
		}
	
	else {
		//printf("RF24L01 receiver enabled\n\r");
		
		RX_POWERUP();  // Power up in receiving mode
		delay_us(150);
		//CE_SetVal();   // Listening for packets
		}
}


void IRQ_OnInterrupt(void)
{
	APP_OnInterrupt();
}

/*!
* \brief Reset the given mask of status bits
* \param flags Flags, one or more of RF24_STATUS_RX_DR, RF24_STATUS_TX_DS, RF24_STATUS_MAX_RT
*/
void RF_ResetStatusIRQ(volatile uint8_t flags) 
{
	delay_us(10);
	RF_CSN_LOW();
	delay_us(10);
	RF_WriteRegister(RF24_STATUS, flags); // reset all IRQ in status register
	delay_us(10);
	RF_CSN_HIGH();
	delay_us(10);
}

/*!
* \brief Send the payload to the Tx FIFO and send it
* \param payload Buffer with payload to send
* \param payloadSize Size of payload buffer
*/
void RF_TxPayload(uint8_t *payload, volatile uint8_t payloadSize)
{
	//RF_WriteRegister(RF24_CONFIG, RF24_PWR_UP|RF24_PRIM_TX); 
	
	RF_CSN_LOW();
	RF_Write(RF24_FLUSH_TX); 
	RF_CSN_HIGH();
	
	RF_WriteRegisterData(RF24_W_TX_PAYLOAD, payload, payloadSize); 

	RF_CE_HIGH(); 
	delay_us(15); 
	//delay_us(70);
	RF_CE_LOW();  
}

/*!
 * \brief Receive the Rx payload from the FIFO and stores it in a buffer.
 * \param payload Pointer to the payload buffer
 * \param payloadSize Size of the payload buffer
 */
void RF_RxPayload(uint8_t *payload, volatile uint8_t payloadSize) 
{
	RF_ReadRegisterData(RF24_R_RX_PAYLOAD, payload, payloadSize); 
}

// Polled send
//void RF_SendDataCh(unsigned char *payload,volatile uint8_t payloadsize,volatile uint8_t channel)
//{
//	RF_TxPayload(payload,payloadsize);
//}

// Polled send
void RF_SendData(unsigned char *payload,volatile uint8_t payloadsize)
{
	RF_TxPayload(payload,payloadsize);
	
	if(RF_GetStatus()&RF24_STATUS_MAX_RT) {
		RF_WriteRegister(RF24_STATUS,RF24_STATUS_MAX_RT);
		}
}

// Switch to tx mode from rx mode
void RF_SendDataCheck(unsigned char *payload,volatile uint8_t payloadsize)
{
	RF_CE_LOW();
	RXTX_POWERDOWN();
	TX_POWERUP();
	delay_us(1000);  // Reduce this number so it works faster with Nano processor
	RF_SendData(payload,payloadsize);
	delay_us(750);  // Allow packet time to transmit - including retries - convert to status check
					// TBD - make this smart so channel is free'd as quickly as possible
	RXTX_POWERDOWN();
	RX_POWERUP();
	delay_us(150);
	RF_CE_HIGH(); 
	delay_us(10);
}

// Switch to tx mode from rx mode
void RF_SendFXOSTransmitCheck(void)
{
	RF_CE_LOW();
	RXTX_POWERDOWN();
	TX_POWERUP();
	delay_us(1000);  // Reduce this number so it works faster with Nano processor
}

// Switch to tx mode from rx mode
void RF_SendFXOSRxCheck(void)
{
	RXTX_POWERDOWN();
	RX_POWERUP();
	delay_us(150);
	RF_CE_HIGH();
	delay_us(10);
}

int RF_SendData_Int(void)
{
	volatile int status;
	
	// Sender routine in main
	if (isrFlag) { // check if we have received an interrupt
		isrFlag = false; // reset interrupt flag
		status = RF_GetStatus();
		if (status&RF24_STATUS_RX_DR) { // data received interrupt
			RF_ResetStatusIRQ(RF24_STATUS_RX_DR); // clear bit
			}
		if (status&RF24_STATUS_TX_DS) { // data sent interrupt
			GreenLedOFF(); // indicate data has been sent
			RF_ResetStatusIRQ(RF24_STATUS_TX_DS); // clear bit
			}
		if (status&RF24_STATUS_MAX_RT) { // retry timeout interrupt
			RF_ResetStatusIRQ(RF24_STATUS_MAX_RT); // clear bit
			}
		}
	return 0;
}

// Non blocking polled receive
int RF_GetData(void)
{
	volatile unsigned char status;
	
	status = RF_GetStatus();
	
	//printf("status=%x  \r",rfstatus);
	if(status&RF24_STATUS_RX_DR) { // data received interrupt
		RF_RxPayload(payload, rf_payload_size); // will reset RX_DR bit
		RF_ResetStatusIRQ(RF24_STATUS_RX_DR|RF24_STATUS_TX_DS|RF24_STATUS_MAX_RT); // make sure we reset all flags. Need to have the pipe number too
		return rf_payload_size;
		}
	if(status&RF24_STATUS_MAX_RT) { // retry timeout interrupt
		RF_ResetStatusIRQ(RF24_STATUS_MAX_RT); // clear bit
		return -2;
		}
	return -1;
}

// Non blocking polled receive
int RF_GetDataWait(volatile int waittime)
{
	volatile unsigned char status;
	long starttime;

	//starttime = mseccounter;
	delay_ms(5);
	//do {
		status = RF_GetStatus();
		if(status&RF24_STATUS_RX_DR) { // data received interrupt
			RF_RxPayload(payload, sizeof(payload)); // will reset RX_DR bit
			RF_ResetStatusIRQ(RF24_STATUS_RX_DR|RF24_STATUS_TX_DS|RF24_STATUS_MAX_RT); // make sure we reset all flags. Need to have the pipe number too
			return sizeof(payload);
			}
		if(status&RF24_STATUS_MAX_RT) { // retry timeout interrupt
			RF_ResetStatusIRQ(RF24_STATUS_MAX_RT); // clear bit
			return -2;
			}
	//	}  while(mseccounter<(starttime+waittime));
	return -1;
}

// Switch to RX mode before getting data then switch back
int RF_GetDataWaitCheck(volatile int waittime)
{
	volatile int payloadbytes;
	
	RXTX_POWERDOWN();
	RX_POWERUP();
	RF_CE_HIGH(); 
	delay_us(150);
	payloadbytes = RF_GetDataWait(waittime);
	RF_CE_LOW(); 
	RXTX_POWERDOWN();
	TX_POWERUP();
	delay_us(150);
	return payloadbytes;
}

void RF_DumpRegisters(int mode)
{
	unsigned char buf[5];
	int i;
	
	if(mode==1) {
#ifdef NOTLEAN
		printf("\n\rRF24_CONFIG=%x\n\r",RF_ReadRegister(RF24_CONFIG));
		printf("RF24_EN_AA=%x\n\r",RF_ReadRegister(RF24_EN_AA));
		printf("RF24_EN_RXADDR=%x\n\r",RF_ReadRegister(RF24_EN_RXADDR));
		printf("RF24_SETUP_AW=%x\n\r",RF_ReadRegister(RF24_SETUP_AW));
		printf("RF24_SETUP_RETR=%x\n\r",RF_ReadRegister(RF24_SETUP_RETR));
		printf("RF24_RF_SETUP=%x\n\r",RF_ReadRegister(RF24_RF_SETUP));
		printf("RF24_STATUS=%x\n\r",RF_ReadRegister(RF24_STATUS));
		printf("RF24_OBSERVE_TX=%x\n\r",RF_ReadRegister(RF24_OBSERVE_TX));
		printf("RF24_RPD=%x\n\r",RF_ReadRegister(RF24_RPD));
		printf("RF24_RX_PW_P0=%x\n\r",RF_ReadRegister(RF24_RX_PW_P0));
		printf("RF24_FIFO_STATUS=%x\n\r",RF_ReadRegister(RF24_FIFO_STATUS));
#endif
		}
	
	printf("RF24_RF_CH=%x\n\r",RF_ReadRegister(RF24_RF_CH));
	printf("RF24_RX_ADDR_P0= ");
	RF_ReadRegisterData(RF24_RX_ADDR_P0, buf, 5);
	for(i=0; i<5; i++) printf("%x ",buf[i]);
	for(i=0; i<4; i++) PutChar(buf[i]);
	printf(" %x ",buf[i]);
	printf("\n\r");
	
	printf("RF24_TX_ADDR   = ");
	RF_ReadRegisterData(RF24_RX_ADDR_P0, buf, 5);
	for(i=0; i<5; i++) printf("%x ",buf[i]);
	for(i=0; i<4; i++) PutChar(buf[i]);
	printf(" %x ",buf[i]);
	printf("\n\r");
}

void RF24L01_Test(void)
{
	volatile char inchar,outstrg[17];
	volatile unsigned char status,i;
	volatile int payloadbytes,loopcnt=0,loopcnt2=0;
	
	printf("\n\rRF24L01 Test A\n\r"); 
	printf("Enter t-transmit, r-receive, e-exit: ");
	//putchar(inchar=getchar());
	inchar = 't';
	GreenLedOFF();
	if(inchar=='t')  RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	else RF24L01_Processor_Init(RFRECEIVER,rf_payload_size,channel_no);
	RF_DumpRegisters(0);
	GreenLedON();
	if(inchar=='t') {
		//printf("\n\rTransmit string: Hello world\n\r");
		while((inchar=GetCharnw3())!='e') {
			//StartTimer();
			GreenLedON();
			sprintf(outstrg,"Hello world %4d",loopcnt++);
			RF_SendData(outstrg,16);
			printf("Sent=%s   ",outstrg);

			delay_ms(1);
			status = RF_GetStatus();
			printf("%x \r",status);
			payloadbytes = RF_GetDataWaitCheck(100);
			payloadbytes = 0;
			if(payloadbytes>0) {
				printf("remote payload=");
				for(i=0; i<payloadbytes; i++)  printf("%c",payload[i]);
				}
			//printf("\n\r");
			//etime = StopTimer();
			//printf(" et=%f\n\r",etime);
			delay_ms(150);
			GreenLedOFF();
			delay_ms(150);
			}
		}
	else if(inchar=='r') {
		printf("\n\rReceive string:\n\r");
		RF_CE_HIGH(); // start transmission
		delay_us(150);
		//StartTimer();
		while((inchar=getchar())!='e') {
			status = RF_GetStatus();
			payloadbytes = RF_GetData();

			printf("s=%x pl=%d ",status,payloadbytes);
			if(payloadbytes>0) {
				for(i=0; i<payloadbytes; i++)  printf("%c",payload[i]);
				printf("\n\r");

				//delay_ms(1);
				sprintf(payload,"return pack %4d",loopcnt2++);
				//RF_Process_Payload(payload);
				}
			else putchar('\r');
			delay_ms(10);
			
			}
		}
}

void RF24L01_ReInit(void)
{
	RF_CE_LOW();
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFRECEIVER,rf_payload_size,channel_no);
	RF_CE_HIGH();
	delay_us(150);
}

// Check if RF24L01 is present and initialize if it is
void rf24L01_Init(volatile char rftype)
{	
	// Enable with default parameters as sender or receiver
	RF24L01_Processor_Init(rftype,rf_payload_size,channel_no);

	// Check if rf24L01 is present by verifying contents of channel number
	// and set default rx or tx state
	if(RF_ReadRegister(RF24_RF_CH)==channel_no)  {
		if(rftype==RFSENDER)  rf_state = RF_STATE_INITIALIZED_TX;
		else {
			RF_CE_HIGH();  // fire up receiver
			rf_state = RF_STATE_INITIALIZED_RX;
			}
		}
	else rf_state = RF_STATE_NOTPRESENT;
}

void ZeroPayload(void)
{
	volatile int i;
	
	for(i=0; i<PAYLOAD_SIZE; i++) payload[i] = 0;
}

void Display_RF_Status(void)
{
	char outstrg[35];
	
	if(rf_state==RF_STATE_NOTPRESENT) {
		printf("RF24L01 Not Active");
#ifdef ST7735LCD
		ST7735_drawString(1, 30,"RF24L01 Not Active",colorsRGB24toRGB565(0,0,0), 1);
#endif
#ifdef SSD1306OLED
	ssd1306_printf(1,0,"RF24L01 InActive");
#endif
		}
	else {
		if(rf_state==RF_STATE_INITIALIZED_RX) {
			printf("RF24L01 RX Mode Active, ");
#ifdef ST7735LCD
			sprintf(outstrg,"RF24L01 RX Mode Live, ");
			ST7735_drawString(1, 30,outstrg,colorsRGB24toRGB565(0,0,0), 1);
#endif
#ifdef SSD1306OLED
			ssd1306_printf(1,0,"RX Live On");
#endif
			}
		else if(rf_state==RF_STATE_INITIALIZED_TX) {
			printf("RF24L01 TX Mode Active, ");
#ifdef ST7735LCD
			sprintf(outstrg,"RF24L01 TX Mode Live, ");
			ST7735_drawString(1, 30,outstrg,colorsRGB24toRGB565(0,0,0), 1);
#endif
#ifdef SSD1306OLED
			ssd1306_printf(1,0,"TX Live On");
#endif
			}
		printf("Ch=%d ID=%d DC=%c",channel_no,device_id,rf_device_class);
#ifdef ST7735LCD
		sprintf(outstrg,"On Ch=%d ID=%d DC=%c",channel_no,device_id,rf_device_class);
		ST7735_drawString(1, 41,outstrg,colorsRGB24toRGB565(0,0,0), 1);
#endif
#ifdef SSD1306OLED
			ssd1306_printf(2,0,"Ch=%d ID=%d DC=%c",channel_no,device_id,rf_device_class);
#endif
		}
}

extern volatile struct imudata {
	int ai[3];
	int gi[3];
	int mi[3];
	int temp;
	float af[3];
	float gf[3];
	float mf[3];
	int tempi;
	int rolli;
	int pitchi;
	int yawi;
	int pressi;
	float tempf;
	float roll;
	float pitch;
	float yaw;
	float press;
	char mget;
} imu;

void RF_Send_10DOFMEMS_Data(void)
{
	volatile int i;
	
	union {
		volatile unsigned char buf[32];
		volatile struct rfmemsdata {
			unsigned long time;
			int accel[3];
			int gyro[3];
			int mag[3];
			int roll,pitch,yaw;
			int temp,press;
		} data;
	} rf;

	//isecond = (int) time.second;
	//imsec = (time.second-isecond) * 1000;
	//rf.data.time = time.hour*3600*1000 + time.minute*60*1000 + isecond*1000 + imsec;
	rf.data.time = 0;  // TBD - real time clock plus clock sync
	
	for(i=0; i<3; i++)  rf.data.accel[i] = imu.ai[i];
	for(i=0; i<3; i++)  rf.data.gyro[i] = imu.gi[i];
	for(i=0; i<3; i++)  rf.data.mag[i] = imu.mi[i];
	rf.data.roll = imu.rolli;
	rf.data.pitch = imu.pitchi;
	rf.data.yaw = imu.yawi;
	rf.data.temp = imu.tempi;
	rf.data.press = imu.pressi;
	
	if((rf_state==RF_STATE_INITIALIZED_TX)&&(rf_mode==RF_STREAM))  RF_SendData(rf.buf,PAYLOAD_SIZE);
	else if(rf_mode==RF_STREAM_ONESHOT) RF_SendDataCheck(rf.buf,PAYLOAD_SIZE);
}

// Switch to RX mode
int RF_GetFXOSDataRXEnable(void)
{
	int payloadbytes;

	RXTX_POWERDOWN();
	RX_POWERUP();
	RF_CE_HIGH();
	Delayusec(150);
}

// Switch to TX mode
int RF_GetFXOSDataTXEnable(void)
{
	RF_CE_LOW();
	RXTX_POWERDOWN();
	TX_POWERUP();
	Delayusec(150);
}

// Diagnostic dump of MEMs and EKF data for RF transfer
void PrintfRF(unsigned char *rffbuf,char linefeed)
{
	unsigned char rfbuf[32];
	int i,hours,minutes;
	float seconds,accelscale,gyroscale,magscale;

	union {
		unsigned char buf[32];
		struct rfmemsdata {
			unsigned long time;
			short int accelx,accely,accelz;
			short int gyrox,gyroy,gyroz;
			short int magx,magy,magz;
			short int roll,pitch,yaw;
			short int temp,press;
		} data;
	} rf;

	// Re-order data
	rfbuf[0] = rffbuf[3]; rfbuf[1] = rffbuf[2]; rfbuf[2] = rffbuf[1]; rfbuf[3] = rffbuf[0];

	rfbuf[4] = rffbuf[5]; rfbuf[5] = rffbuf[4];
	rfbuf[6] = rffbuf[7]; rfbuf[7] = rffbuf[6];
	rfbuf[8] = rffbuf[9]; rfbuf[9] = rffbuf[8];

	rfbuf[10] = rffbuf[11]; rfbuf[11] = rffbuf[10];
	rfbuf[12] = rffbuf[13]; rfbuf[13] = rffbuf[12];
	rfbuf[14] = rffbuf[15]; rfbuf[15] = rffbuf[14];

	rfbuf[16] = rffbuf[17]; rfbuf[17] = rffbuf[16];
	rfbuf[18] = rffbuf[19]; rfbuf[19] = rffbuf[18];
	rfbuf[20] = rffbuf[21]; rfbuf[21] = rffbuf[20];

	rfbuf[22] = rffbuf[23]; rfbuf[23] = rffbuf[22];
	rfbuf[24] = rffbuf[25]; rfbuf[25] = rffbuf[24];
	rfbuf[26] = rffbuf[27]; rfbuf[27] = rffbuf[26];

	rfbuf[28] = rffbuf[29]; rfbuf[29] = rffbuf[28];
	rfbuf[30] = rffbuf[31]; rfbuf[31] = rffbuf[30];

	// Set scaling constants
	accelscale = 1.0;
	//gyroscale = CalculateGyroScale(sensorrange);
	gyroscale = 1.0;
	//magscale = CalculateMagScale(sensorrange);
	magscale = 1.0;

	//printf("\n\r");
	for(i=0; i<32; i++) {
		rf.buf[i] = rffbuf[i];
		//printf("%x ",rf.buf[i]);
		}

	//printf("\n\r");
	if(0) {
		//printf("%ld ",rf.data.time);
		//printf("%d %d %d ",rf.data.accelx,rf.data.accely,rf.data.accelz);
		//printf("%d %d %d ",rf.data.gyrox,rf.data.gyroy,rf.data.gyroz);
		//printf("%d %d %d ",rf.data.magx,rf.data.magy,rf.data.magz);
		//printf("%d %d %d ",rf.data.roll,rf.data.pitch,rf.data.yaw);
		//printf("%d %d\n\r",rf.data.temp,rf.data.press);
		}
	else {
		//printf("%ld ",rf.data.time);
		hours = rf.data.time/(long)(3600*1000);
		//printf("h=%d ",hours);
		rf.data.time = rf.data.time - ((long)hours*3600*1000);
		//printf("%ld ",rf.data.time);
		minutes = rf.data.time/((long)60*1000);
		//printf("m=%d ",minutes);
		rf.data.time = rf.data.time - ((long)minutes*60*1000);
		//printf("%ld ",rf.data.time);
		seconds = ((float) rf.data.time)/1000.0;
		printf("%2d:%2d:%6.4f ",hours,minutes,seconds);
		printf("%6.3f %6.3f %6.3f ", (float)rf.data.accelx*0.0039*accelscale,(float)rf.data.accely*0.0039*accelscale,(float)rf.data.accelz*0.0039*accelscale);
		printf("%6.1f %6.1f %6.1f ",((float) rf.data.gyrox)*250.0*gyroscale/32768.0,((float) rf.data.gyroy)*250.0*gyroscale/32768.0,((float) rf.data.gyroz)*250.0*gyroscale/32768.0);
		printf("%5.3f %5.3f %5.3f ",((float) rf.data.magx)*1.3*magscale/2048.0,((float) rf.data.magy)*1.3*magscale/2048.0,((float) rf.data.magz)*1.3*magscale/2048.0);
		//printf("%5.1f %5.1f %5.1f \r",(float)rf.data.roll/10.0,(float)rf.data.pitch/10.0,(float)rf.data.yaw/10.0);
		//printf("%6.2f %6.4f\n\r",(float)rf.data.temp/100.0,(float)rf.data.press/1000.0);
	}
	if(linefeed==YES)  printf("\n\r");
	else printf("\r");
}

void Printf_PicoMEMs(unsigned char *rfbuf,char linefeed,char datamode)
{
	//unsigned char rfbuf[32];
	int i,hours,minutes;
	float seconds,accelscale,gyroscale,magscale;

	union {
		unsigned char buf[32];
		struct rfmemsdata {
			unsigned long time;
			short int accelx,accely,accelz;
			short int gyrox,gyroy,gyroz;
			short int magx,magy,magz;
			short int roll,pitch,yaw;
			short int temp,press;
		} data;
	} rf;

	// Set scaling constants
	accelscale = 1.0;
	//gyroscale = CalculateGyroScale(sensorrange);
	gyroscale = 1.0;
	//magscale = CalculateMagScale(sensorrange);
	magscale = 1.0;

	//printf("\n\r");
	for(i=0; i<32; i++) {
		rf.buf[i] = rfbuf[i];
		//printf("%x ",rf.buf[i]);
		}

	//printf("\n\r");
	if(0) {
		//printf("%ld ",rf.data.time);
		//printf("%d %d %d ",rf.data.accelx,rf.data.accely,rf.data.accelz);
		//printf("%d %d %d ",rf.data.gyrox,rf.data.gyroy,rf.data.gyroz);
		//printf("%d %d %d ",rf.data.magx,rf.data.magy,rf.data.magz);
		//printf("%d %d %d ",rf.data.roll,rf.data.pitch,rf.data.yaw);
		//printf("%d %d\n\r",rf.data.temp,rf.data.press);
	}
	else {
		//printf("%ld ",rf.data.time);
		hours = rf.data.time/(long)(3600*1000);
		//printf("h=%d ",hours);
		rf.data.time = rf.data.time - ((long)hours*3600*1000);
		//printf("%ld ",rf.data.time);
		minutes = rf.data.time/((long)60*1000);
		//printf("m=%d ",minutes);
		rf.data.time = rf.data.time - ((long)minutes*60*1000);
		//printf("%ld ",rf.data.time);
		seconds = ((float) rf.data.time)/1000.0;
		//printf("%2d:%2d:%6.4f ",hours,minutes,seconds);
		if(datamode==RAWDATA) {
			printf("%6.3f %6.3f %6.3f  ", (float)rf.data.accelx/2048.0,(float)rf.data.accely/2048.0,(float)rf.data.accelz/2048.0);
			printf("%6.1f %6.1f %6.1f  ",((float) rf.data.gyrox)*gyroscale/16.384,((float) rf.data.gyroy)*gyroscale/16.384,((float) rf.data.gyroz)*gyroscale/16.384);
			//printf("%5.3f %5.3f %5.3f ",((float) rf.data.magx)*(1200.0/3095.0),((float) rf.data.magy)*(1200.0/3095.0),((float) rf.data.magz)*(1200.0/3095.0));
			printf("%4d %4d %4d ",rf.data.magx,rf.data.magy,rf.data.magz);
			}
		else {
			printf("%5.1f %5.1f %5.1f \r",(float)rf.data.roll/100.0,(float)rf.data.pitch/100.0,(float)rf.data.yaw/100.0);
			}
		//printf("%6.2f %6.4f\n\r",(float)rf.data.temp/100.0,(float)rf.data.press/1000.0);
	}
	if(linefeed==YES) printf("\n\r");
	//else printf("\r");
}

void Printf_PicoM305(payload,linefeed)
{
	//unsigned char rfbuf[32];
	int i,hours,minutes;
	float seconds,accelscale,gyroscale,magscale;

	union {
		unsigned char buf[32];
		struct rfmemsdata {
			unsigned long time;
			short int accelx,accely,accelz;
			short int gyrox,gyroy,gyroz;
			short int magx,magy,magz;
			short int roll,pitch,yaw;
			short int temp,press;
		} data;
	} rf;

	// Set scaling constants
	accelscale = 1.0;
	//gyroscale = CalculateGyroScale(sensorrange);
	gyroscale = 1.0;
	//magscale = CalculateMagScale(sensorrange);
	magscale = 1.0;

	//printf("\n\r");
	for(i=0; i<32; i++) {
		rf.buf[i] = rfbuf[i];
		//printf("%x ",rf.buf[i]);
	}

	//printf("\n\r");
	if(0) {
		//printf("%ld ",rf.data.time);
		//printf("%d %d %d ",rf.data.accelx,rf.data.accely,rf.data.accelz);
		//printf("%d %d %d ",rf.data.gyrox,rf.data.gyroy,rf.data.gyroz);
		//printf("%d %d %d ",rf.data.magx,rf.data.magy,rf.data.magz);
		//printf("%d %d %d ",rf.data.roll,rf.data.pitch,rf.data.yaw);
		//printf("%d %d\n\r",rf.data.temp,rf.data.press);
	}
	else {
		//printf("%ld ",rf.data.time);
		hours = rf.data.time/(long)(3600*1000);
		//printf("h=%d ",hours);
		rf.data.time = rf.data.time - ((long)hours*3600*1000);
		//printf("%ld ",rf.data.time);
		minutes = rf.data.time/((long)60*1000);
		//printf("m=%d ",minutes);
		rf.data.time = rf.data.time - ((long)minutes*60*1000);
		//printf("%ld ",rf.data.time);
		seconds = ((float) rf.data.time)/1000.0;
		//printf("%2d:%2d:%6.4f ",hours,minutes,seconds);
		printf("%6.3f %6.3f %6.3f  ", (float)rf.data.accelx/2048.0,(float)rf.data.accely/2048.0,(float)rf.data.accelz/2048.0);
		printf("%6.1f %6.1f %6.1f  ",((float) rf.data.gyrox)*gyroscale/16.384,((float) rf.data.gyroy)*gyroscale/16.384,((float) rf.data.gyroz)*gyroscale/16.384);
		//printf("%5.3f %5.3f %5.3f ",((float) rf.data.magx)*(1200.0/3095.0),((float) rf.data.magy)*(1200.0/3095.0),((float) rf.data.magz)*(1200.0/3095.0));
		printf("%4d %4d %4d ",rf.data.magx,rf.data.magy,rf.data.magz);
		//printf("%5.1f %5.1f %5.1f \r",(float)rf.data.roll/10.0,(float)rf.data.pitch/10.0,(float)rf.data.yaw/10.0);
		//printf("%6.2f %6.4f\n\r",(float)rf.data.temp/100.0,(float)rf.data.press/1000.0);
	}
	if(linefeed==YES) printf("\n\r");
	//else printf("\r");
}

void RF_SendDataCh(unsigned char channel)
{
	channel_no = channel;
	strcpy(rfbuf,"src");
	rfbuf[3] = channel;
	RF_SendData(rfbuf,rf_payload_size);
	printf("Sent=%c%c%c%d   ",rfbuf[0],rfbuf[1],rfbuf[2],rfbuf[3]);
	Delaymsec(1);
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	RF_DumpRegisters(0);
}

void RF_SendDataId(unsigned char id)
{
	strcpy(rfbuf,"sri");
	device_id = rfbuf[3] = id;
	RF_SendData(rfbuf,rf_payload_size);
	printf("Sent=%c%c%c%d   ",rfbuf[0],rfbuf[1],rfbuf[2],rfbuf[3]);
	Delaymsec(2);
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	RF_DumpRegisters(0);
}

void RF_SendDataPayload(unsigned char payloadsize)
{
	strcpy(rfbuf,"srp");
	rf_payload_size = rfbuf[3] = payloadsize;
	RF_SendData(rfbuf,payloadsize);
	printf("Sent=%c%c%c%d   ",rfbuf[0],rfbuf[1],rfbuf[2],rfbuf[3]);
	Delaymsec(1);
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	RF_DumpRegisters(0);
}

int RF24L01_Enumeration(char mode)
{
	unsigned char device_id_now,rfbuf[32];
	int i,id,foundcnt=0,payloadbytes;

	device_id_now = device_id;
	for(id=0; id<MAX_DEVICES; id++) {
		rfbuf[0] = 'i';
		device_id = id;
		RXTX_POWERDOWN();

		RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
		Delaymsec(1);
		RF_SendData(rfbuf,32);
		//if(mode==YES) printf("\n\rSent=i to %d  ",device_id);
		Delaymsec(1);

		payloadbytes = RF_GetDataWaitCheck(25);
		if(payloadbytes>0) {
			deviceaddress[foundcnt+1] = id;
			foundcnt++;
			if(mode==YES) printf("\n\rSensor node at %d returned=",id);
			i = 0;
			while((payload[i]!=0)&&(payload[i]!='&')&&(payload[i]!='$')&&(i<32)) PutChar(payload[i++]);
			//for(i=0; i<16; i++)  printf("%c",payload[i]);
			}

		//else if(mode==YES) printf("no receiver at %d",id);
		}
	deviceaddress[0] = foundcnt;
	device_id = device_id_now;

	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	return foundcnt;
}

int RF_Get_Joystick(void)
{
	unsigned char device_id_now,rfbuf[32];
	int i,payloadbytes;

	device_id_now = device_id;  // save current device

	// todo - if current device is selected device do re-init

	rfbuf[0] = 'g';
	rfbuf[1] = 'a';
	device_id = 3;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	Delaymsec(1);
	RF_SendData(rfbuf,32);
	Delaymsec(1);
	payloadbytes = RF_GetDataWaitCheck(25);
	if(payloadbytes>0) {
		printf("CJ34b at %d =",3);
		i = 0;
		while((i<32)&&(payload[i]!='&'))  printf("%c",payload[i++]);
		//for(i=0; i<22; i++)  printf("%c",payload[i]);
		printf("\r");
		}
	device_id = device_id_now;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
}

void RF_GetAll_IMU8420_Data(unsigned char id)
{
	unsigned char device_id_now,outstrg[32];
	int payloadbytes,i;

	device_id_now = device_id;  // save current device

	sprintf(outstrg,"ga                 ");
	device_id = id;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	//RD_DumpRegisters(0);
	Delaymsec(1);
	RF_SendData(outstrg,32);
	Delaymsec(1);
	payloadbytes = RF_GetDataWaitCheck(30);
	if(payloadbytes>0) PrintfRF(payload,YES);
	else {
		printf("0 ");
		for(i=0; i<9; i++) printf("0.0 ");
		}
	device_id = device_id_now;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
}

int RF_GetAll_PICOA_Data(unsigned char id,char linefeed,char datamode)
{
	unsigned char device_id_now,outstrg[32];
	int payloadbytes,i;

	device_id_now = device_id;  // save current device

	sprintf(outstrg,"ga                 ");
	device_id = id;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	//RD_DumpRegisters(0);
	Delaymsec(1);
	RF_SendData(outstrg,32);
	//if(mode==YES) printf("\n\rSent=i to %d  ",device_id);
	Delaymsec(1);
	payloadbytes = RF_GetDataWaitCheck(25);
	if(payloadbytes>0) {
		if(datamode==RAWDATA) printf("9 ");
		else printf("3");
		Printf_PicoMEMs(payload,linefeed,datamode);
		}
	else {
		printf("0 ");
		if(datamode==RAWDATA) for(i=0; i<9; i++) printf("0.0 ");
		else for(i=0; i<3; i++) printf("0.0 ");
		}

	device_id = device_id_now;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);

	return payloadbytes;
}

unsigned char fxosbuffer[180];

// Move FXOS data stored in large char buffer to five FXOS structures
void Parse_FXOS_RFbuffer(void)
{
	int i,bufptr=0;
	union {
		unsigned char bytebuf[2];
		short int intvalue;
	} data;

	for(i=0; i<5; i++) {
		// Finger joint mcp
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		mcp[i].mxi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		mcp[i].myi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		mcp[i].mzi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		mcp[i].axi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		mcp[i].ayi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		mcp[i].azi = data.intvalue;

		// Finger joint pip
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		pip[i].mxi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		pip[i].myi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		pip[i].mzi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		pip[i].axi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		pip[i].ayi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		pip[i].azi = data.intvalue;

		// Finger joint dip
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		dip[i].mxi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		dip[i].myi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		dip[i].mzi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		dip[i].axi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		dip[i].ayi = data.intvalue;
		data.bytebuf[0] = fxosbuffer[bufptr++];
		data.bytebuf[1] = fxosbuffer[bufptr++];
		dip[i].azi = data.intvalue;
		}
}

void FXOS8700_Convert2Float(int fingerid)
{
	mcp[fingerid].mx = (float)mcp[fingerid].mxi/MAGSCALE10;
	mcp[fingerid].my = (float)mcp[fingerid].myi/MAGSCALE10;
	mcp[fingerid].mz = (float)mcp[fingerid].mzi/MAGSCALE10;
	mcp[fingerid].ax = (float)mcp[fingerid].axi*ACCELSCALE;
	mcp[fingerid].ay = (float)mcp[fingerid].ayi*ACCELSCALE;
	mcp[fingerid].az = (float)mcp[fingerid].azi*ACCELSCALE;

	pip[fingerid].mx = (float)pip[fingerid].mxi/MAGSCALE10;
	pip[fingerid].my = (float)pip[fingerid].myi/MAGSCALE10;
	pip[fingerid].mz = (float)pip[fingerid].mzi/MAGSCALE10;
	pip[fingerid].ax = (float)pip[fingerid].axi*ACCELSCALE;
	pip[fingerid].ay = (float)pip[fingerid].ayi*ACCELSCALE;
	pip[fingerid].az = (float)pip[fingerid].azi*ACCELSCALE;

	dip[fingerid].mx = (float)dip[fingerid].mxi/MAGSCALE10;
	dip[fingerid].my = (float)dip[fingerid].myi/MAGSCALE10;
	dip[fingerid].mz = (float)dip[fingerid].mzi/MAGSCALE10;
	dip[fingerid].ax = (float)dip[fingerid].axi*ACCELSCALE;
	dip[fingerid].ay = (float)dip[fingerid].ayi*ACCELSCALE;
	dip[fingerid].az = (float)dip[fingerid].azi*ACCELSCALE;

}

// Total number of bytes to transfer is 15*6*2=180 bytes which
// maps into 5.625 packets or 6 packets.  PicoM305 will send
// 6 32byte packets one after the other
void RF_GetAll_PICOM305_Data(unsigned char id,char linefeed,char displaymode)
{
	unsigned char device_id_now,outstrg[32];
	int payloadbytes,i,j,fxosptr=0;

	device_id_now = device_id;  // save current device

	sprintf(outstrg,"gf                 ");
	device_id = id;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	//RD_DumpRegisters(0);
	Delaymsec(1);
	RF_SendData(outstrg,32);
	//if(mode==YES) printf("\n\rSent=i to %d  ",device_id);
	Delaymsec(1);
	RF_GetFXOSDataRXEnable();  // Switch to receiver
	for(i=0; i<6; i++) {
		payloadbytes = RF_GetDataWait(25);
		if(payloadbytes)
			for(j=0; j<32; j++) fxosbuffer[fxosptr++] = payload[j];   // packet good - fill big receive buffer
		else for(j=0; j<32; j++) fxosbuffer[fxosptr++] = payload[j];  // lost packet
		}

	Parse_FXOS_RFbuffer();

	for(i=0; i<5; i++) 	{
		FXOS8700_Convert2Float(i);
		if((displaymode=='b')||(displaymode=='a')) printf("f%d a: d=%6.3f %6.3f %6.3f p=%6.3f %6.3f %6.3f m=%6.3f %6.3f %6.3f \n\r",i,
					dip[i].ax,dip[i].ay,dip[i].az,pip[i].ax,pip[i].ay,pip[i].az,mcp[i].ax,mcp[i].ay,mcp[i].az);
		if((displaymode=='b')||(displaymode=='m')) printf("f%d m: d=%6.1f %6.1f %6.1f p=%6.1f %6.1f %6.1f m=%6.1f %6.1f %6.1f \n\r",i,
					dip[i].mx,dip[i].my,dip[i].mz,pip[i].mx,pip[i].my,pip[i].mz,mcp[i].mx,mcp[i].my,mcp[i].mz);
		}


	RF_GetFXOSDataTXEnable(); // Turn rf channel around

	device_id = device_id_now;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);

	return payloadbytes;
}

void RF_GetAll_PICOD_Data(unsigned char id,char linefeed)
{
	unsigned char device_id_now,outstrg[32];
	int payloadbytes,i;

	device_id_now = device_id;  // save current device

	sprintf(outstrg,"ga                 ");
	device_id = id;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
	//RD_DumpRegisters(0);
	Delaymsec(1);
	RF_SendData(outstrg,32);
	//if(mode==YES) printf("\n\rSent=i to %d  ",device_id);
	Delaymsec(1);
	payloadbytes = RF_GetDataWaitCheck(25);
	if(payloadbytes>0) {
		i = 0;
		while((i<32)&&(payload[i]!='$')) {
			PutChar(payload[i]);
			i++;
			}
		}
	else {
		for(i=0; i<2; i++) printf(" 0.0");
		}
	if(linefeed==YES)  printf("\n\r");
	else PutChar('\r');

	device_id = device_id_now;
	RXTX_POWERDOWN();
	RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
}

void RF_GetAll_CJ34b_Data(void)
{
	unsigned char outstrg[32];
	int payloadbytes;

	sprintf(outstrg,"ga                 ");
	RF_SendData(outstrg,32);
	outstrg[16] = 0;
	Delayusec(500);
	payloadbytes = RF_GetDataWaitCheck(30);
	if(payloadbytes>0) PrintfRF(payload,YES);
}

// CJ34b Joystick test utility
void CJ34b_Test(char inchar)
{
	unsigned char device_id_now,rfbuf[32],bytevalue;
	char inchar2,inchar3,deviceswitch=NO,instrg[5];
	int i=0,intvalue;

	if(device_id!=3) {
		device_id_now = device_id;
		device_id = 3;
		deviceswitch = YES;
		RXTX_POWERDOWN();
		RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
		Delaymsec(1);
		}
	rfbuf[0] = 's';
	rfbuf[1] = inchar;
	PutChar(inchar2=GetChar());
	rfbuf[2] = inchar2;
	switch (inchar) {
		case 'l':
			if((inchar2=='b')||(inchar2=='n')) RF_SendData(rfbuf,32);
			if((inchar2=='1')||(inchar2=='2')||(inchar2=='3')) {
				PutChar(inchar3=GetChar());
				rfbuf[3] = inchar3;
				RF_SendData(rfbuf,32);
				}
			break;
		case 'r':
			if((inchar2=='c')||(inchar2=='i')||(inchar2=='p')) {
				while((instrg[i++]=GetChar())!=CRR) ;
				instrg[i] = 0;
				sscanf(instrg,"%d",&intvalue);
				rfbuf[3] = bytevalue = intvalue;
				RF_SendData(rfbuf,32);
				}
			else if(inchar2=='r') RF_SendData(rfbuf,32);
			break;
		case 's':
			RF_SendData(rfbuf,32);
			break;
		}

	if(deviceswitch==YES) {
		Delaymsec(1);
		device_id = device_id_now;
		RXTX_POWERDOWN();
		RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
		}
}

// Smart LCD test utility
#ifdef SMARTLCDRF
void SmartRF_LCD_Test(void)
{
	unsigned char device_id_now,bytevalue;
	char inchar,inchar2,inchar3,deviceswitch=NO,rfbuf[32],instrg[5],outstrg[12];
	int i=0,intvalue,payloadbytes;

	for(i=0; i<32; i++) rfbuf[i] = 0x00;
	i = 0;
	if(device_id!=4) {
		device_id_now = device_id;
		device_id = 4;
		deviceswitch = YES;
		RXTX_POWERDOWN();
		RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
		Delaymsec(1);
		}

	PutChar(inchar=GetChar());
	rfbuf[0] = inchar;
	PutChar(inchar2=GetChar());
	rfbuf[1] = inchar2;
	if(inchar!='g') {
		PutChar(inchar3=GetChar());
		rfbuf[2] = inchar3;
		}

	// Send data
	if(inchar=='d') {
		rfbuf[1] -= 0x30;
		rfbuf[2] -= 0x30;
		while((inchar=GetChar())!=CRR) {
			PutChar(inchar);
			rfbuf[3+i] = inchar;
			i++;
			}
		rfbuf[3+i] = 0x00;
		i = 0;
		while(rfbuf[3+i]!=0) {
			outstrg[i] = rfbuf[3+i];
			i++;
			}
		outstrg[i] = 0x00;
		printf("\n\rLCD: %d %d %s\n\r",rfbuf[1],rfbuf[2],outstrg);
		for(i=0; i<32; i++) printf("%x ",rfbuf[i]);

		RF_SendData(rfbuf,32);
		}
	else if(inchar=='g') {
		RF_SendData(rfbuf,32);
		Delaymsec(1);
		payloadbytes = RF_GetDataWaitCheck(25);
		if(payloadbytes>0) {
			printf("\n\rSmartRFLCD at %d =",3);
			i = 0;
			while((i<32)&&(payload[i]!='&'))  printf("%c",payload[i++]);
			//for(i=0; i<22; i++)  printf("%c",payload[i]);
			printf("\r");
			}
		}
	// s case
	else if(inchar=='s') {
		switch (inchar2) {
				case 'l':
					if((inchar2=='b')||(inchar2=='n')) RF_SendData(rfbuf,32);
					if((inchar2=='1')||(inchar2=='2')) {
						PutChar(inchar3=GetChar());
						rfbuf[3] = inchar3;
						RF_SendData(rfbuf,32);
						}
					break;
				case 'r':
					if((inchar2=='c')||(inchar2=='i')||(inchar2=='p')) {
						while((instrg[i++]=GetChar())!=CRR) ;
						instrg[i] = 0;
						sscanf(instrg,"%d",&intvalue);
						rfbuf[3] = bytevalue = intvalue;
						RF_SendData(rfbuf,32);
						}
					else if(inchar2=='r') RF_SendData(rfbuf,32);
					break;
				case 's':
					RF_SendData(rfbuf,32);
					break;
				case 'd':
					RF_SendData(rfbuf,32);
					break;
				}
			}

	if(deviceswitch==YES) {
		Delaymsec(1);
		device_id = device_id_now;
		RXTX_POWERDOWN();
		RF24L01_Processor_Init(RFSENDER,rf_payload_size,channel_no);
		}
}
#endif
// Function to send commands to remote Pico processor using RF24L01
void RF24L01_Process_Command(void)
{
	unsigned char id;
	char inchar,inchar2,outstrg[32],device_id_now;
	int i,payloadbytes;

	printf("\n\rEnter Command [i,n,sc,sp,si]: ");
	while((inchar=GetnwChar())!='e') {
		if(inchar) {
			printf("%c",inchar);
			sprintf(outstrg,"                ");
			outstrg[0] = inchar;
			switch (inchar) {
				// Network scan
				case 'n':
					printf("\n\rSensor nodes found=%d",RF24L01_Enumeration(YES));
					break;
				// Identify remote sensor node
				case 'i':
					//RF_SendData(outstrg,rf_payload_size);
					RF_SendData(outstrg,32);
					outstrg[16] = 0;
					printf("\n\rSent=i  ");
					Delaymsec(2);
					payloadbytes = RF_GetDataWaitCheck(100);
					if(payloadbytes>0) {
						printf("  return %d=",payloadbytes);
						i = 0;
						while((payload[i]!='$')&&(i<32))  printf("%c",payload[i++]);
						}
					break;
				// Set commands
				case 's':
					inchar2 = GetChar();
					printf("%c\n\r",inchar2);
					if(inchar2=='c') RF_SendDataCh(3);
					else if(inchar2=='p') RF_SendDataPayload(32);
					else if(inchar2=='i') RF_SendDataId(2);
					else if(inchar2=='d') {
						RF_SendDataId(DEVICE_ID);
						Delaymsec(10);
						RF_SendDataCh(CHANNEL_NO);
						}
					break;
				// Get commands
				case 'g':
					PutChar(inchar2=GetChar());
					outstrg[1] = inchar2;
					if(inchar2=='c') {	}
					else if(inchar2=='p') {	}
					else if(inchar2=='i') {	}
					else if(inchar2=='a') {
						//RF_SendData(outstrg,rf_payload_size);
						printf("\n\r");
						RF_GetAll_IMU8420_Data(8);
						}
					break;
				}
			printf("\n\rEnter Command: ");
			}
		}
	//  Shut radio down before exiting
	RXTX_POWERDOWN();
	RF_CE_LOW();
}

// Check for commands coming in over RF24L01 link.
// Only called if link hardware present and set to receive mode
// Any query commands turn the link around but restore it to receive
// before returning.
bool RF24L01_Process_Commands2(void)
{
	volatile char outstrg[20];
	volatile int i,payloadbytes;
	volatile rf_mess=false;
	
	//printf("\n\rrf status=%x\n\r",RF_GetStatus());
	
	payloadbytes = RF_GetData();

	//if(payloadbytes>=0) printf("\n\rpayload=%d\n\r",payloadbytes);

	if(payloadbytes>0) {
		rf_mess = true;
		switch (payload[0]) {
			case 'i':
				ZeroPayload();
#ifdef PAELLAM04TARGET
#ifdef SSD1306OLED
				strcpy(payload,"NanoMP04L V1.0 9DOF OLED$");
#else
				strcpy(payload,"NanoMP04 V1.0 9DOF$");
#endif
#endif
#ifdef NANOMP07TARGET
#ifdef SSD1306OLED
				strcpy(payload,"NanoMP07L V1.0 OLED$");
#else
				strcpy(payload,"NanoMP07 V1.0$");
#endif
#endif
#ifdef PAELLAM30TARGET
#ifdef SSD1306OLED
				strcpy(payload,"NanoMP30L V1.0 9DOF OLED$");
#else
				strcpy(payload,"NanoMP30 V1.0 9DOF$");
#endif
#endif
				RF_SendDataCheck(payload,rf_payload_size);
				break;
			// Set commands
			case 's':
				switch (payload[1]) {
					case 'i':
						//RF_SetDigital(payload);
						break;
					case 'd':
						//RF_SetDAC();
						break;
					case 't':
						//RF_SetTime();
						break;
					case 'm':
						//RF_SetMotor(payload);
						break;
					case 'r':
						if(payload[2]=='c')  {
							channel_no = payload[3];
							Flash_Store_Parameters(QUIET);
							RF24L01_ReInit();
							//printf("rx'd sc=%d\n\r",payload[3]);
							//RD_DumpRegisters(0);
							}
						else if(payload[2]=='i') {
							device_id = payload[3];
							Flash_Store_Parameters(QUIET);
							RF24L01_ReInit();
							//printf("rx'd si=%d  %d\n\r",payload[3],channel_no);
							//RD_DumpRegisters(0);
							}
						else if(payload[2]=='p') {
							rf_payload_size = payload[3];
							Flash_Store_Parameters(QUIET);
							RF24L01_ReInit();
							//printf("rx'd sp=%d\n\r",payload[3]);
							//RD_DumpRegisters(0);
							}
						break;
					case 'l':
						if(payload[2]=='b') ledblink = YES;
						if(payload[2]=='o') {
							ledblink = NO; 
							GreenLedON();
							}
						if(payload[2]=='f') {
							ledblink = NO;
							GreenLedOFF();
							}
						break;
					case 's':
						//RF_SetStreamMode(payload);  // sets stream rate and sensor mix
						break;
					}
				break;
			// Get commands
			case 'g':
				switch (payload[1]) {
					case 't':
						//RF_GetTime();
						break;
					case 'a':
						rf_mode = RF_STREAM_ONESHOT;
						if(memsstate!=ACTIVESTATE)  Get_10DOFMEMsData('s');
						RF_Send_10DOFMEMS_Data();
						rf_mode = RF_STREAM;
						break;
#ifdef FXOS8700SENSOR
					case 'f':
						rf_mode = RF_STREAM_ONESHOT;
						//if(memsstate!=ACTIVESTATE)  FXOS8700_Display_All(QUIET);
						RF_Send_FXOS_Data();
						rf_mode = RF_STREAM;
						break;
#endif
#ifdef FSI7013SENSOR
					case 'h':
						//strcpy(payload,"rh=");
						strcpy(payload,ftoa(si7013.humidity,2,6));
						strcat(payload," ");
						strcat(payload,ftoa(si7013.temperature,2,6));
						if(BMPstate) {
							strcat(payload," ");
							strcat(payload,ftoa(bmppressure,4,8));
							strcat(payload," ");
							strcat(payload,ftoa(bmptemperature,2,6));
							}
						strcat(payload,"$");
						rf_mode = RF_STREAM_ONESHOT;
						if((rf_state==RF_STATE_INITIALIZED_TX)&&(rf_mode==RF_STREAM))  RF_SendData(payload,PAYLOAD_SIZE);
						else if(rf_mode==RF_STREAM_ONESHOT) RF_SendDataCheck(payload,PAYLOAD_SIZE);
						rf_mode = RF_STREAM;
						break;
#endif
						case 'd':
						//RF_GetDigital(payload);
						break;
					case 'm':
						//RF_GetMEMsData();
						break;
					case 'e':
						//RF_GetEKFData();
						break;
					case 'l':
						//RF_GetLogParameters();
						break;
					}
				break;
			case 'd':
				for(i=0; i<payloadbytes; i++)  PutChar(payload[i]);
				printf("\n\r");
			break;
		}
	}
	return rf_mess;
}

// Check if RF24L01 is present and initialize if it is
void rf24L01_init(char rftype)
{
	// Enable with default parameters as sender or receiver
	RF24L01_Processor_Init(rftype,rf_payload_size,channel_no);

	//LED(GREENLED,ON);

	// Check if rf24L01 is present by verifying contents of channel number
	// and set default rx or tx state
	if(RF_ReadRegister(RF24_RF_CH)==CHANNEL_NO)  {
		if(rftype==RFSENDER)  rf_state = RF_STATE_INITIALIZED_TX;
		else {
			RF_CE_HIGH();  // fire up receiver
			rf_state = RF_STATE_INITIALIZED_RX;
		}
	}
	else rf_state = RF_STATE_NOTPRESENT;

	//LED(REDLED,ON);
}


